<?php

namespace Lc\ShopBundle\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Lc\ShopBundle\Context\OrderProductInterface;
use Lc\ShopBundle\Context\PriceInterface;
use Lc\ShopBundle\Context\ProductInterface;
use Lc\ShopBundle\Context\ProductPropertyInterface;
use Lc\ShopBundle\Context\SortableInterface;
use Lc\ShopBundle\Services\Price;

/**
 * @ORM\MappedSuperclass()
 */
abstract class Product extends AbstractEntity implements SortableInterface, ProductPropertyInterface, PriceInterface
{
        use SortableTrait;
        use ProductPropertyTrait;

        /**
         * @ORM\ManyToOne(targetEntity="Lc\ShopBundle\Context\ProductFamilyInterface", inversedBy="products")
         * @ORM\JoinColumn(nullable=false)
         */
        protected $productFamily;

        /**
         * @ORM\Column(type="string", length=255, nullable=true)
         */
        protected $title;

        /**
         * @ORM\OneToMany(targetEntity="Lc\ShopBundle\Context\OrderProductInterface", mappedBy="product", orphanRemoval=true, cascade={"remove"})
         */
        protected $orderProducts ;

        public function __construct()
        {
                $this->orderProducts = new ArrayCollection() ;
        }

        public function getPriceInherited()
        {
                if($this->price) {
                        return $this->price;
                }
                else {
                        return $this->productFamily->getPrice();
                }
        }

        public function getUnitInherited()
        {
                if($this->unit) {
                        return $this->unit;
                }
                else{
                        return $this->productFamily->getUnit();
                }
        }

        public function getTitleInherited()
        {
                if($this->title){
                        return $this->title;
                }
                else{
                        return $this->productFamily->getTitle();
                }
        }

        public function getQuantityInherited()
        {
                if($this->quantity) {
                        return $this->quantity;
                }
                else{
                        return $this->productFamily->getQuantity();
                }
        }

        public function getQuantityLabelInherited()
        {
                $quantity = $this->getQuantityInherited() ;
                $unit = $this->getUnitInherited() ;
                return $quantity.$unit->getWordingShort() ;
        }

        public function getAvailableQuantityInherited()
        {
                if($this->productFamily->getBehaviorCountStock()) {
                        return $this->availableQuantity;
                }
                else{
                        return $this->productFamily->getAvailableQuantity();
                }
        }

        public function getTaxRateInherited()
        {
                if($this->productFamily->getTaxRate()) {
                        return $this->productFamily->getTaxRate();
                }
                else{
                        return $this->productFamily->getMerchant()->getTaxRate()->getValue();
                }
        }

        public function getProductFamily(): ?ProductFamily
        {
                return $this->productFamily;
        }

        public function setProductFamily(?ProductFamily $productFamily): self
        {
                $this->productFamily = $productFamily;

                return $this;
        }

        public function getTitle(): ?string
        {
                return $this->title;
        }

        public function setTitle(?string $title): self
        {
                $this->title = $title;

                return $this;
        }

        /**
         * @return Collection|OrderProductInterface[]
         */
        public function getOrderProducts(): Collection
        {
                return $this->orderProducts;
        }

        public function addOrderProduct(OrderProductInterface $orderProduct): self
        {
                if (!$this->orderProducts->contains($orderProduct)) {
                        $this->orderProducts[] = $orderProduct;
                        $orderProduct->setProduct($this);
                }

                return $this;
        }

        public function removeOrderProduct(OrderProductInterface $orderProduct): self
        {
                if ($this->orderProducts->contains($orderProduct)) {
                        $this->orderProducts->removeElement($orderProduct);
                        // set the owning side to null (unless already changed)
                        if ($orderProduct->getProduct() === $this) {
                                $orderProduct->setProduct(null);
                        }
                }

                return $this;
        }
}
