let mixinPrice = {
    data() {
        return Object.assign({
            price: null,
            priceWithTax: null,
            buyingPrice: null,
            buyingPriceWithTax: null,
            buyingPriceByRefUnit: null,
            buyingPriceByRefUnitWithTax: null,
            differentSupplierTaxRate: null,
            multiplyingFactor: null,
            priceByRefUnit: null,
            priceByRefUnitWithTax: null,
            taxRate: null,
            supplierTaxRate: null
        }, window.mixinPriceValues);

    },
    computed: {
        taxRateValue: function () {
            if (this.taxRate) {
                return this.taxRatesList[this.taxRate]['value'];
            } else if (this.taxRate == null || this.taxRate == "undefined") {
                return this.taxRatesList['default']['value'];
            } else {
                log('ERREUR : pas de taxRate')
            }
        },
        supplierTaxRateValue: function () {
            if (this.supplierTaxRate) {
                return this.taxRatesList[this.supplierTaxRate]['value'];
            } else if (this.supplierTaxRate == null || this.supplierTaxRate == "undefined") {
                return this.taxRateValue;
            } else {
                log('ERREUR : pas de supplier taxRate')
            }
        },

    },
    mounted: function () {
    },
    methods: {
        init: function () {
            this.buyingPriceUpdated();
            this.priceUpdated();
            this.unitUpdated();
        },

        priceUpdated: function () {
            if(this.price) {
                this.setPriceWithTax();
                this.setMultiplyingFactor();
                this.setPriceByRefUnit();
                this.setPriceByRefUnitWithTax();
            }
        },


        buyingPriceUpdated: function () {
            if(this.buyingPrice) {
                this.setBuyingPriceWithTax();
                this.setMultiplyingFactor();
                this.setBuyingPriceByRefUnit();
                this.setBuyingPriceByRefUnitWithTax();
            }
        },

        unitUpdated: function () {
            if(this.unit) {
                this.setPriceByRefUnit();
                this.setPriceByRefUnitWithTax();
                //this.setBuyingPriceByRefUnit();
                //this.setBuyingPriceByRefUnitWithTax();
            }
        },
        quantityUpdated: function () {
            if(this.quantity) {
                this.setPriceByRefUnit();
                this.setPriceByRefUnitWithTax();
               // this.setBuyingPriceByRefUnit();
                // this.setBuyingPriceByRefUnitWithTax();
            }
        },

//Toutes les fonctions ci-dessous ne font que mettre à jour les price et buyingPrice
        buyingPriceWithTaxUpdated: function () {
            this.setBuyingPriceFromBuyingPriceWithTax();
            this.buyingPriceUpdated();
        },
        buyingPriceByRefUnitUpdated: function () {
            this.setBuyingPriceFromBuyingPriceByRefUnit();
            this.buyingPriceUpdated();
        },
        buyingPriceByRefUnitWithTaxUpdated: function () {
            this.setBuyingPriceByRefUnitFromBuyingPriceByRefUnitWithTax();
            this.buyingPriceByRefUnitUpdated();
        },
        multiplyingFactorUpdated: function () {
            this.setPriceFromMultiplyingFactor();
            this.priceUpdated();
        },
        priceWithTaxUpdated: function () {
            this.setPriceFromPriceWithTax();
            this.priceUpdated();
        },
        priceByRefUnitUpdated: function () {
            this.setPriceFromPriceByRefUnit();
            this.priceUpdated();
        },
        priceByRefUnitWithTaxUpdated: function () {
            this.setPriceByRefUnitFromTax();
            this.priceByRefUnitUpdated();
        },


        //SETTER
        setBuyingPriceFromBuyingPriceByRefUnit:function(){
            this.buyingPrice = parseFloat((this.buyingPriceByRefUnit * this.quantityValue) / this.unitCoefficient).toFixed(2);
        },
        setBuyingPriceFromBuyingPriceWithTax: function () {
            if (this.buyingPriceWithTax) {
                this.buyingPriceWithTax = parseFloat(this.buyingPriceWithTax.replace(',', '.')).toFixed(2);
                this.buyingPrice = getPrice(this.buyingPriceWithTax, this.supplierTaxRateValue);
            }
        },
        setBuyingPriceWithTax: function () {
            if (this.buyingPrice) {
                this.buyingPrice = parseFloat(this.buyingPrice.replace(',', '.')).toFixed(3);
                this.buyingPriceWithTax = getPriceWithTax(this.buyingPrice, this.supplierTaxRateValue);
            }
        },
        setBuyingPriceByRefUnitFromBuyingPriceByRefUnitWithTax: function () {
            if (this.buyingPriceByRefUnitWithTax) {
                this.buyingPriceByRefUnitWithTax = parseFloat(this.buyingPriceByRefUnitWithTax.replace(',', '.')).toFixed(2);
                this.buyingPriceByRefUnit = getPrice(this.buyingPriceByRefUnitWithTax, this.supplierTaxRateValue);
            }
        },
        setBuyingPriceByRefUnit: function () {
            if (this.unitCoefficient && this.quantityValue) {
                this.buyingPriceByRefUnit = parseFloat((this.buyingPriceValue / this.quantityValue) * this.unitCoefficient).toFixed(2);
            }
        },
        setBuyingPriceByRefUnitWithTax: function () {
            if (this.buyingPriceByRefUnit) {
                this.buyingPriceByRefUnit = parseFloat(this.buyingPriceByRefUnit.replace(',', '.')).toFixed(3);
                this.buyingPriceByRefUnitWithTax = getPriceWithTax(this.buyingPriceByRefUnit, this.supplierTaxRateValue);
            }
        },
        setMultiplyingFactor: function () {
            if (this.priceWithTax || this.buyingPrice) {
                this.multiplyingFactor = parseFloat(this.priceWithTaxValue / this.buyingPriceValue).toFixed(3);
            }
        },
        setPriceByRefUnitFromTax: function () {
            if (this.priceByRefUnitWithTax) {
                this.priceByRefUnitWithTax = parseFloat(this.priceByRefUnitWithTax.replace(',', '.')).toFixed(2);
                this.priceByRefUnit = getPrice(this.priceByRefUnitWithTax, this.taxRateValue);
            }
        },
        setPriceByRefUnit: function () {
            if (this.unitCoefficient && this.quantityValue) {
                this.priceByRefUnit = parseFloat((this.priceValue / this.quantityValue) * this.unitCoefficient).toFixed(2);
            }
        },
        setPriceByRefUnitWithTax: function () {
            if (this.unitCoefficient && this.quantityValue) {
                this.priceByRefUnitWithTax = parseFloat((this.priceWithTaxValue / this.quantityValue) * this.unitCoefficient).toFixed(2);
            }
        },

        setPriceFromMultiplyingFactor: function () {
            this.price =  getPrice(this.buyingPriceValue * this.multiplyingFactor, this.taxRateValue);
        },
        setPriceFromPriceByRefUnit: function () {
            this.price = parseFloat((this.priceByRefUnit * this.quantityValue) / this.unitCoefficient).toFixed(2);

        },
        setPriceFromPriceWithTax: function () {
            if (typeof this.priceWithTax != "number") {
                this.priceWithTax = parseFloat(this.priceWithTax.replace(',', '.'));
            }
            this.priceWithTax = this.priceWithTax.toFixed(2);
            this.price = getPrice(this.priceWithTax, this.taxRateValue);
        },
        setPriceWithTax() {
            if (this.price) {
                this.price = parseFloat(this.price.replace(',', '.')).toFixed(3);
                this.priceWithTax = getPriceWithTax(this.price, this.taxRateValue);
            }
        },
    },
    watch: {}

};

let mixinUnit = {
    data() {
        return Object.assign(
            {
                quantity: null,
                unit: null
            }, window.mixinUnitValues);
    },
    computed: {
        unitWording: function () {
            if (this.unit) {
                return this.unitsList[this.unit]['wording'];
            } else if (this.unitValue) {
                return this.unitsList[this.unitValue]['wording'];
            } else {
                return '';
            }
        },
        unitCoefficient: function () {
            if (this.unit) {
                return this.unitsList[this.unit]['coefficient'];
            } else if (this.unitValue) {
                return this.unitsList[this.unitValue]['coefficient'];
            } else {
                return 0;
            }
        },
        unitReference: function () {
            if (this.unit) {
                return this.unitsList[this.unitsList[this.unit]['unitReference']]['wording'];
            } else if (this.unitValue) {
                return this.unitsList[this.unitsList[this.unitValue]['unitReference']]['wording'];
            } else {
                return '';
            }
        }

    }
};
let mixinTemplate = {
    data() {
        return {
            templateRender: null,
        }
    },
    render(h) {
        if (!this.templateRender) {
            return h('div', 'loading...');
        } else { // If there is a template, I'll show it
            return this.templateRender();
        }
    },
    watch: {
        // Every time the template prop changes, I recompile it to update the DOM
        template: {
            immediate: true, // makes the watcher fire on first render, too.
            handler() {
                if (this.template) {
                    var res = Vue.compile(this.template);

                    this.templateRender = res.render;

                    // staticRenderFns belong into $options,
                    // appearantly
                    this.$options.staticRenderFns = []

                    // clean the cache of static elements
                    // this is a cache with the results from the staticRenderFns
                    this._staticTrees = []

                    // Fill it with the new staticRenderFns
                    for (var i in res.staticRenderFns) {
                        //staticRenderFns.push(res.staticRenderFns[i]);
                        this.$options.staticRenderFns.push(res.staticRenderFns[i])
                    }

                }
            }
        }
    }
}
