// Reference array sent to dynamic staticRenderFns
var staticRenderFns = [];


Vue.component('product-unit-price', {
    mixins: [mixinUnit, mixinPrice, mixinTemplate],
    props: ['template', 'keyForm'],
    data() {
        return Object.assign({}, window.productUnitPriceValues);

    },
    computed: {
        buyingPriceValue: function () {
            return this.buyingPrice;
        },
        buyingPriceWithTaxValue: function () {
            return this.buyingPriceWithTax;
        },
        priceWithTaxValue: function () {
            return this.priceWithTax;
        },
        priceValue: function () {
            return this.price;
        },
        quantityValue: function () {
            return this.quantity;
        },
        unitValue: function () {
            return this.unit;
        }
    },
    mounted: function () {
        this.init();
    },
    watch: {
        taxRate: function () {
            this.setBuyingPriceWithTax();
            this.setPriceWithTax();
            this.setMultiplyingFactor();
            this.setPriceByUnitRefWithTax();
        },
        supplierTaxRate: function () {
            this.setBuyingPriceWithTax();
        }
    }
});

Vue.component('product-form', {
    mixins: [mixinUnit, mixinPrice, mixinTemplate],
    props: ['template', 'keyForm', 'productFamily'],
    computed: {
        taxRate: function () {
            return this.productFamily.taxRate;
        },
        supplierTaxRate: function () {
            return this.productFamily.supplierTaxRate;
        },
        buyingPriceValue: function () {
            if (this.buyingPrice) return this.buyingPrice;
            else return this.productFamily.buyingPrice;
        },
        buyingPriceWithTaxValue: function () {
            if (this.buyingPriceWithTax) return this.buyingPriceWithTax;
            else return this.productFamily.buyingPriceWithTax;
        },
        priceWithTaxValue: function () {
            if (this.priceWithTax) return this.priceWithTax;
            else return this.productFamily.priceWithTax;
        },
        priceValue: function () {
            if (this.price) return this.price;
            else return this.productFamily.price;
        },
        quantityValue: function () {
            if (this.quantity) return this.quantity;
            else return this.productFamily.quantity;
        },
        unitValue: function () {
            if (this.unit) return this.unit;
            else return this.productFamily.unit;
        },
        propertyExpirationDateFormated: function () {

            if (this.propertyExpirationDate) return getDateFormatted(this.propertyExpirationDate, '-')
            else return getDateFormatted(this.productFamily.propertyExpirationDate, '-')
        }
    },
    data() {
        return Object.assign(
            {
                title: null,
                fieldToUpdate: ['title', 'unit', 'quantity', 'price'],
                price: null,
                priceWithTax: null,
                buyingPrice: null,
                multiplyingFactor: null,
                availableQuantityDefault: null,
                availableQuantity: null,
                propertyExpirationDate: null,

                titleInherited: false,
                priceInherited: false,
                priceWithTaxInherited: false,
                unitInherited: false,
                quantityInherited: false,
                multiplyingFactorInherited: false,
                buyingPriceInherited: false,
                buyingPriceWithTaxInherited: false,
                buyingPriceByRefUnitInherited: false,
                buyingPriceByRefUnitWithTaxInherited: false,
                priceByRefUnitInherited:false,
                priceByRefUnitWithTaxInherited:false,
                expirationDateInherited: false,
                availableQuantityInherited: false,
                availableQuantityDefaultInherited: false,
                propertyExpirationDateInherited: false

            }, window.productForm[this.keyForm])
    },
    mounted: function () {
         //INIT VAR
        updateSortableProducts();
        $(this.$el).find('select').each(function (i, select) {
            setSelect2($(select));
        });


        //METHOD
        this.updateProductForm();
        this.updateProductView();
    },
    methods: {
        titleUpdated:function(){},
        availableQuantityUpdated:function(){},
        availableQuantityDefaultUpdated:function(){},
        propertyExpirationDateUpdated:function(){},
        updateLine: function(){
            this.init();
        },
        updateProductForm: function () {
            this.init();
        },

        updateProductView: function () {


            /*if (this.unitValue == 'piece') {
                $('.priceByRefUnit').hide();
            } else {
                $('.priceByRefUnit').show();
            }
            if (this.productFamily.behaviorCountStock == 'by-product-family' || this.productFamily.behaviorCountStock == 'by-quantity') {
                $('.products-collection-table').find('.stock').hide();
            } else {
                $('.products-collection-table').find('.stock').show();
                $('#product-item-' + this.keyForm).find('.stock').html(this.availableQuantity + ' / ' + this.unitReference);
            }*/
        },
        saveProductForm: function () {
            this.updateProductView();
            $('#form-product-modal-' + this.keyForm).modal('hide');
        },
        deleteProductForm: function () {
            if (confirm('Êtes-vous sur de cette action ?')) {
                this.$parent.formProductArray.splice(this.keyForm, 1);
            }
        },
    },
    watch: {
        /*titleInherited: function (val) {

            if (val) this.title = null;
        },
        unitInherited: function (val) {
            if (val) this.unit = null;
        },
        quantityInherited: function (val) {
            if (val) this.quantity = null;
        },
        priceInherited: function (val) {
            if (val) this.price = null;
        },*/
    }
});


appProductFamily = new Vue({
    el: '#lc-product-family-edit',
    delimiters: ['${', '}'],
    computed: {
        productFamily: function () {
            return {
                'title': this.title,
                'behaviorCountStock': this.behaviorCountStock,
                'behaviorBuyingPrice': this.behaviorBuyingPrice,
                'displayPriceByRefUnit': this.displayPriceByRefUnit,
                'unit': this.$refs.productUnitPrice.unit,
                'unitWording': this.$refs.productUnitPrice.unitWording,
                'quantity': this.$refs.productUnitPrice.quantity,
                'price': this.$refs.productUnitPrice.price,
                'priceWithTax': this.$refs.productUnitPrice.priceWithTax,
                'buyingPrice': this.$refs.productUnitPrice.buyingPrice,
                'buyingPriceWithTax': this.$refs.productUnitPrice.buyingPriceWithTax,
                'buyingPriceByRefUnit': this.$refs.productUnitPrice.buyingPriceByRefUnit,
                'buyingPriceByRefUnitWithTax': this.$refs.productUnitPrice.buyingPriceByRefUnitWithTax,
                'priceByRefUnit': this.$refs.productUnitPrice.priceByRefUnit,
                'priceByRefUnitWithTax': this.$refs.productUnitPrice.priceByRefUnitWithTax,
                'taxRate': this.$refs.productUnitPrice.taxRate,
                'supplierTaxRate': this.$refs.productUnitPrice.supplierTaxRate,
                'multiplyingFactor': this.$refs.productUnitPrice.multiplyingFactor,
                'taxRateValue': this.$refs.productUnitPrice.taxRateValue,
                'propertyExpirationDate': this.propertyExpirationDate,
                'behaviorExpirationDate': this.behaviorExpirationDate
            };
        },
        unitReference: function () {
            this.getUnitReference();

        }
    },
    data() {

        return Object.assign(
            {
                indexFormProduct: 0,
                title: null,
                activeProducts: false,
                isNovelty: null,
                isOrganic: null,
                propertyExpirationDate: null,
                behaviorCountStock: null,
                behaviorBuyingPrice:null,
                displayPriceByRefUnit: false,
                typeExpirationDate: null,
                behaviorExpirationDate: null,
                propertyAllergens: null,
                propertyOrganicLabelActive: false,
                propertyNoveltyExpirationDateActive: false,

                formProductArray: [],
                currentSection: 'general',
                sectionsArray: [
                    {
                        name: 'general',
                        nameDisplay: 'Général'
                    },
                    {
                        name: 'price',
                        nameDisplay: 'Prix '
                    },
                    {
                        name: 'products',
                        nameDisplay: 'Déclinaisons'
                    },
                    {
                        name: 'stock',
                        nameDisplay: 'Stock'
                    },
                    {
                        name: 'property',
                        nameDisplay: 'Caractéristiques'
                    },

                    {
                        name: 'note',
                        nameDisplay: 'Note interne'
                    }
                ]
            }, window.appProductFamilyValues);
    },
    mounted: function () {
        $(".vuejs-checkbox-switch input").each(function () {
            setBootstrapSwitch($(this));
        });
    },
    methods: {

        changeSection: function (section) {
            this.updateChild();
            this.currentSection = section.name;
            $(".vuejs-checkbox-switch input").each(function () {
                setBootstrapSwitch($(this));
            });
        },
        addProductForm: function () {
            var $collectionHolder = $('table.products-collection-table');
            var prototype = $collectionHolder.data('prototype');
            var newForm = prototype;
            newForm = newForm.replace(/__name__/g, this.indexFormProduct);
            this.formProductArray.push(newForm);
            this.indexFormProduct++;

            //updateSortableProducts();
        },

        updateChild: function () {
            if (typeof this.$refs.productForm !== 'undefined') {
                for (i = 0; i < this.$refs.productForm.length; i++) {
                    this.$refs.productForm[i].updateProductForm();
                    this.$refs.productForm[i].updateProductView();
                }
            }
        },
        getUnitReference: function () {
            if (typeof this.$refs.productUnitPrice !== 'undefined') {
                return this.$refs.productUnitPrice.unitReference;
            }
        }
    },
    watch: {
        title: function () {
            this.updateChild()
        },
    }
});


$(window).on('load', function () {
    for (i = 0; i < window.formProductTemplate.length; i++) {
        appProductFamily.formProductArray.push(formProductTemplate[i]);
        appProductFamily.indexFormProduct++;
    }
});
